      subroutine btwan_qp(ub,nb,pb,u,n,p,e,ne,q,gwtb,mass,wdth)
c----------------------------------------------------------------------c
c Returns a four vector array e,
c         e=ub*Gamma_Wtb^n*u*(-g_mn + q_m*q_n/mv2)/(q^2-mv2)
c and its four momentum q with the anomalous Wtb coupling,
c see K. Kolodziej, Phys. Lett. B584 (2004) 89, hep-ph/0312168, Eq.(4)
c Input: 
c        ub - spinor of a b-quark,
c        u - spinor of a t-quark,
c        pb,p - four momenta,
c        nb,ne,n - numbers of polarization states
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 10 Feb., 2011
c 11 April, 2011: error in a(2,2,1) corrected
c 6 March, 2020: 
c quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer nb,ne,n

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: pb,p,q,pe
      complex(dp) :: gwtb(2,4)
      complex(qp), dimension(4,nb) :: ub
      complex(qp), dimension(4,n) :: u
      complex(qp), dimension(0:3,ne) :: e

c Local variables:

      integer ib,ie,i,l

      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: f1l,f1r,f2l,f2r,mv2
      complex(qp) :: den
      complex(qp), dimension(0:3,2,2) :: a,d

      f1r=gwtb(1,1)
      f1l=gwtb(2,1)
      f2r=gwtb(1,2)
      f2l=gwtb(2,2)

      mv2=cmplx(mass**2,-mass*wdth,dp)

      q=pb+p

      den=cmplx(q(0)**2-q(1)**2-q(2)**2-q(3)**2,0.q0,qp)-mv2

      pe=-q ! Vertex is parametrized in terms of outgoing W four momentum

      a(0,1,:)=(/cmplx(pe(3),0.q0,qp),cmplx(pe(1),-pe(2),qp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),qp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.q0,pe(2),qp),cmplx(pe(0)-pe(3),0.q0,qp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.q0,qp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.q0,-pe(1),qp),cmplx(0.q0,-pe(0)+pe(3),qp)/)
      a(2,2,:)=(/cmplx(0.q0,pe(0)+pe(3),qp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.q0,qp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      ie=0

      do i=1,n
       do ib=1,nb
        ie=ie+1
        do l=0,3
         e(l,ie)=f2l*(ub(1,ib)*(a(l,1,1)*u(1,i)+a(l,1,2)*u(2,i))
     &               +ub(2,ib)*(a(l,2,1)*u(1,i)+a(l,2,2)*u(2,i)))
     &          +f2r*(ub(3,ib)*(d(l,1,1)*u(3,i)+d(l,1,2)*u(4,i))
     &               +ub(4,ib)*(d(l,2,1)*u(3,i)+d(l,2,2)*u(4,i)))
        end do
        e(0,ie)=e(0,ie)+f1l*(ub(3,ib)*u(1,i)+ub(4,ib)*u(2,i))
     &                 +f1r*(ub(1,ib)*u(3,i)+ub(2,ib)*u(4,i))
        e(1,ie)=e(1,ie)-f1l*(ub(3,ib)*u(2,i)+ub(4,ib)*u(1,i))
     &                 +f1r*(ub(1,ib)*u(4,i)+ub(2,ib)*u(3,i))
        e(2,ie)=e(2,ie)-f1l*(-ub(3,ib)*u(2,i)+ub(4,ib)*u(1,i))*icom
     &                 +f1r*(-ub(1,ib)*u(4,i)+ub(2,ib)*u(3,i))*icom
        e(3,ie)=e(3,ie)-f1l*(ub(3,ib)*u(1,i)-ub(4,ib)*u(2,i))
     &                 +f1r*(ub(1,ib)*u(3,i)-ub(2,ib)*u(4,i))
        e(:,ie)=-e(:,ie)/den
        e(:,ie)=e(:,ie)-(q(0)*e(0,ie)-q(1)*e(1,ie)-q(2)*e(2,ie)
     &                  -q(3)*e(3,ie))/mv2*q
       end do
      end do

      end
