      subroutine nan_qp(bu,nb,pb,e,ne,pe,u,n,p,ip,e0,mu,wdth)
c----------------------------------------------------------------------c
c Returns matrix elements with an effective nucleon-nucleon-photon 
c coupling
c    Gamma^n=e0*[F_1(q^2)*gamma^n+F_2(q^2)*i*sigma^nm*q_m]
c Factor 1/(2m) included in F_2. Based on a routine fvfan.f.
c Input: 
c        bu - spinor,
c        e -  polarization vector,
c        pb - four momentum of bu,
c        pe - four momentum of incoming e,
c        nb,ne,n - numbers of polarization states,
c        ip=+1/-1 for u representing a particle/an antiparticle,
c        e0 - electric charge.
c        mu,wdth - mass and width (wdth not used) of a nucleon 
c                  reprezented by u and ub
c Output: u - fermion array,
c         p - its momentum.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 19 Feb./13 March, 2014
c 6 March, 2020: 
c quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer :: nb,ne,n,ip

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)

      real(dp) :: e0,mu,wdth
      complex(dp) :: f1,f2
      real(qp), dimension(0:3) :: pb,pe,p
      complex(qp), dimension(4,nb) :: bu
      complex(qp), dimension(0:3,ne) :: e
      complex(qp), dimension(4,n) :: u

c Local variables:

      integer ib,ie,i

      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: f1l,f1r,f2l,f2r,mf,mf2
      complex(qp) :: den
      complex(qp), dimension(4) :: u1,pp,pm
      complex(qp), dimension(2,2) :: b,c,ae,de
      complex(qp), dimension(0:3,2,2) :: a,d
      real(dp) :: q2

      q2=pe(0)**2-pe(1)**2-pe(2)**2-pe(3)**2

      call nuclff_phok(mu,q2,f1,f2)

      f1r=e0*f1
      f1l=f1r
      f2r=e0*f2
      f2l=f2r

      if (ip == -1) then
       f1r=conjg(f1r)
       f1l=conjg(f1l)
       f2r=conjg(f2r)
       f2l=conjg(f2l)
      end if

      mf2=cmplx(mu**2,0.d0,dp)

      mf=sqrt(mf2)

      p=pb+pe

      pp=ip*(/cmplx(p(0)-p(3),0.q0,qp),   ! (1,1)
     &        cmplx(-p(1),-p(2),qp),      ! (2,1)
     &        cmplx(-p(1),p(2),qp),       ! (1,2)
     &        cmplx(p(0)+p(3),0.q0,qp) /) ! (2,2)

      pm=(/pp(4),-pp(2),-pp(3),pp(1)/)

      den=pp(1)*pp(4)-pp(3)*pp(2)-mf2

      a(0,1,:)=(/cmplx(pe(3),0.q0,qp),cmplx(pe(1),-pe(2),qp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),qp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.q0,pe(2),qp),cmplx(pe(0)-pe(3),0.q0,qp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.q0,qp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.q0,-pe(1),qp),cmplx(0.q0,-pe(0)+pe(3),qp)/)
      a(2,2,:)=(/cmplx(0.q0,pe(0)+pe(3),qp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.q0,qp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      i=0

      do ie=1,ne
       b(1,:)=(/e(0,ie)-e(3,ie),-e(1,ie)+icom*e(2,ie)/)
       b(2,:)=(/-e(1,ie)-icom*e(2,ie),e(0,ie)+e(3,ie)/)
       c(1,:)=(/b(2,2),-b(1,2)/)
       c(2,:)=(/-b(2,1),b(1,1)/)
       ae=(a(0,:,:)*e(0,ie)-a(1,:,:)*e(1,ie)
     &    -a(2,:,:)*e(2,ie)-a(3,:,:)*e(3,ie))
       de=(d(0,:,:)*e(0,ie)-d(1,:,:)*e(1,ie)
     &    -d(2,:,:)*e(2,ie)-d(3,:,:)*e(3,ie))
       do ib=1,nb
        i=i+1
        u1(:2)=f2l*matmul(bu(:2,ib),ae)+f1l*matmul(bu(3:,ib),c)
        u1(3:)=f1r*matmul(bu(:2,ib),b)+f2r*matmul(bu(3:,ib),de)
        u(:,i)=(/u1(3)*pm(1)+u1(4)*pm(2)+mf*u1(1),
     &           u1(3)*pm(3)+u1(4)*pm(4)+mf*u1(2),
     &           u1(1)*pp(1)+u1(2)*pp(2)+mf*u1(3),
     &           u1(1)*pp(3)+u1(2)*pp(4)+mf*u1(4)/)
       end do
      end do

      u=u/den

      end




